
# q-expansion of newform 8730.2.a.v, downloaded from the LMFDB on 15 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8730
weight = 2
poly_data = [-2, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1, 0], [0, 0], [-1, 0], [0, 0], [2, -2], [-2, 1], [2, 1], [-4, -1], [0, 3], [-2, 6], [-6, -2], [-6, -1], [2, -3], [0, -4], [-4, -6], [4, -4], [4, 2], [-6, 2], [0, 9], [4, 3], [-4, 0], [2, 2], [-8, 4], [-2, 0], [-1, 0], [4, -8], [-8, 4], [-4, 2], [-6, 8], [6, 0], [8, 0], [0, 4], [2, 1], [-4, 7], [-10, -4], [0, 8], [2, -9], [0, -8], [-20, -4], [2, -12], [8, -6], [10, -9], [4, 0], [-10, -4], [-6, -12], [-16, -2], [12, -11], [-8, -8], [-10, -2], [-22, 2], [-10, -13], [-12, 1], [-4, 8], [0, 8], [-18, 3], [16, -1], [-6, 12], [-12, 10], [18, -3], [2, 5], [20, -4], [6, 12], [8, -6], [-12, 3], [-2, -4], [10, 14], [-12, 9], [-22, 6], [0, 22], [-14, -3], [10, -4], [20, 3], [0, 10], [2, 1], [-4, 4], [-8, 19], [14, 0], [-6, -12], [-18, 7], [2, -8], [-4, -12], [2, 0], [-24, -2], [14, -14], [-4, -16], [8, -18], [30, -6], [-2, 20], [-12, 8], [2, 14], [-18, 6], [12, 8], [10, 2], [-12, -16], [-20, -7], [-12, -14], [-4, 8], [2, 10], [8, 21], [-18, 9], [12, -14], [-30, 8], [-4, 10], [-18, -13], [8, -16], [22, 4], [28, -4], [14, 20], [-12, -11], [-10, 0], [24, 0], [-18, -6], [6, -22], [-20, -19], [-16, 8], [42, -3], [-8, 28], [-16, -2], [2, 24], [8, 6], [18, -8], [24, 4], [-20, -4], [4, -8], [16, 6], [10, -12], [2, 9], [-20, -5], [-2, -14], [42, -2], [20, -21], [24, 10], [8, 28], [-2, -5], [-2, -9], [10, -26], [-16, 0], [20, 22], [-18, 4], [34, 2], [20, -14], [6, -10], [16, -28], [36, 4], [14, 18], [-4, 9], [10, -3], [26, -2], [4, 21], [-24, -15], [-22, -16], [-10, -10], [0, 21], [-24, -1], [24, 3], [20, 25], [-20, -6], [2, -37], [-4, -36], [18, 16], [16, 30], [-6, -7], [22, 10], [22, 2], [-2, -23], [-24, -7], [28, -18], [2, 22], [-14, -24], [14, -24], [12, -4], [14, 33], [16, -12], [-18, -2], [-4, 30], [18, 4], [24, -26], [2, -16], [6, -6], [2, 22], [-4, 12], [-20, -12], [-18, -17], [22, 29], [24, -12], [2, 16], [-2, -28], [8, -1], [-12, -24], [-4, 21], [-40, 4], [6, 14], [28, 1], [22, -4], [52, 2], [14, 1], [-18, 0], [18, -16], [54, 0], [8, 25], [-4, 0], [48, -2], [-10, 34], [-24, 24], [-4, -24], [34, 12], [-18, -14], [4, -20], [22, 19], [-28, 3], [-4, 24], [26, 28], [4, -16], [28, -12], [-28, -7], [-30, 14], [62, 6], [26, 20], [8, -16], [38, 0], [18, 15], [-16, 0], [-34, 1], [16, -8], [24, -20], [-42, 13], [-10, 10], [-4, -12], [-8, 16], [12, 20], [-38, -4], [-44, -10], [38, 6], [22, -13], [24, -9], [36, 6], [-26, -20], [-38, 16], [14, 6], [4, -3], [48, 8], [4, -13], [8, -28], [2, 2], [-18, -12], [-52, -3], [-16, -38], [-20, -26], [28, -18], [-24, -12], [-34, -27], [-10, 38], [-8, 19], [-18, 18], [12, -4], [-56, 2], [2, -5], [56, -8], [44, -12], [-20, -24], [-24, 16], [-34, 14], [6, -31], [-18, 24], [-30, 2], [-32, -20], [-42, -16], [-26, 22], [16, 43], [26, -16], [26, -7], [24, 14], [-30, -22], [20, -32], [12, 8], [-8, 40], [24, -20], [-42, 6], [-34, -8], [2, 10], [-48, 5], [-64, -8], [-16, -48], [-10, 6], [0, 14], [28, -27], [-58, -4], [6, -28], [-2, 38], [-10, 11], [-38, 16], [26, 2], [-10, 30], [-26, 14], [-42, 23], [-4, 16], [8, 48], [38, 28], [-4, 48], [-4, -48], [20, 16], [-10, -4], [-24, -6], [28, 24], [60, 7], [22, -20], [16, 36], [66, -2], [12, -30], [-34, 34], [-24, 29], [-46, 12], [10, -42], [-32, 25], [-36, -24], [-2, -6], [-20, -44], [-36, -17], [-30, -18], [10, 1], [-8, 38], [0, 4], [-26, 2], [42, -26], [6, -11], [-60, -8], [-4, 19], [-40, -27], [28, -20], [-26, -48], [26, -9], [-24, 44], [-10, 2], [42, 6], [60, -11], [2, 26], [38, 11], [34, 3], [32, 40], [4, 62], [-18, 26], [6, 16], [-74, -6], [24, -26], [46, -6], [-4, -4], [-50, -21], [32, -21], [68, 7], [2, -54], [4, -36], [-54, 16], [-22, 32], [-36, -16], [18, -26], [14, -18], [-28, -29], [-32, 12], [22, 36], [16, 6], [-22, -32], [-38, 32], [-16, 6], [-16, 28], [-32, 35], [56, 20], [-2, -14], [-48, -10], [6, 52], [14, -26], [4, -1], [0, -27], [-60, -24], [-20, -20], [42, -18], [76, -2], [28, -25], [-74, 4], [-10, -33], [74, 0], [-10, -8], [6, -3], [12, 6], [10, -25], [-12, 39], [4, 60], [-48, -12], [42, 17], [-44, 20], [-24, 5], [-6, 0], [-18, 0], [-12, -20], [-8, 26], [-28, -5], [4, -44], [16, -32], [26, -5], [20, -7], [-2, -40], [-58, 20], [-18, 35], [-36, -14], [10, 20], [4, -8], [0, -8], [34, 26], [74, -6], [-72, -13], [-12, 0], [-54, 32], [-36, 36], [-24, -48], [-4, -39], [-54, 8], [-42, 40], [-4, 44], [12, -6], [14, -1], [24, -17], [-48, -8], [-38, -13], [-8, 29], [-28, -32], [2, -12], [-18, 20], [52, 0], [-2, -35], [20, -10], [60, 8], [46, 0], [-28, 36], [36, 10], [28, -24], [-6, 27], [-6, 15], [-6, -38], [-26, 6], [8, 29], [-82, -6], [24, -34], [82, 19], [6, -51], [44, 33], [62, -10], [42, 38], [24, -5], [16, 21], [-30, -16], [50, 13], [48, 13], [-28, 19], [18, -26], [-58, -14], [-68, -16], [-26, 12], [42, -15], [68, -12], [46, 27], [46, -11], [-36, 39], [-24, 28], [-4, -20], [-22, 8], [36, -46], [-54, 34], [100, 4], [58, -2], [30, 40], [-28, -10], [-48, -34], [76, 24], [-44, 44], [-26, 42], [-18, 34], [-38, -32], [-20, -48], [-56, -4], [-64, -28], [68, -20], [-10, 56], [-30, -8], [40, 36], [18, -68], [-20, 30], [-12, -28], [-18, 23], [-44, -8], [100, -7], [-80, 14], [-2, -26]]
